/***********************************************************************************************************************
 *
 * blueBill Core - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://kenai.com/hg/bluebill~core-src
 *
 **********************************************************************************************************************/
package it.tidalwave.semantic.rss;

import it.tidalwave.role.SimpleComposite;
import it.tidalwave.role.spi.DefaultSimpleComposite;
import javax.annotation.Nonnull;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import it.tidalwave.util.Id;
import it.tidalwave.util.Key;
import it.tidalwave.util.Finder;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.util.spi.SimpleFinderSupport;
import it.tidalwave.semantic.EntityWithProperties;
import it.tidalwave.semantic.document.Document;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class RssFeed extends EntityWithProperties<RssFeed>
  {
    public RssFeed (final @Nonnull Id id)
      {
        super(id);
      }

    private RssFeed (final @Nonnull Id id,
                     final @Nonnull Map<Key<?>, Object> propertyMap,
                     final @Nonnull Object[] capabilities) 
      {
        super(id, propertyMap, capabilities);
      } 

    @Override @Nonnull
    protected RssFeed clone (final @Nonnull Id id,
                             final @Nonnull Map<Key<?>, Object> propertyMap,
                             final @Nonnull Object[] capabilities)
      {
        return new RssFeed(id, propertyMap, capabilities);
      }

    @Override @Nonnull
    protected Lookup createLookup() 
      {
        final Finder<Document> finder = new SimpleFinderSupport<Document>("messages") 
          {
            @Override @Nonnull
            protected List<? extends Document> computeResults() 
              {
                try 
                  {
                    return get(RssVocabulary.MESSAGES);
                  }
                catch (NotFoundException e)
                  {
                    return Collections.emptyList();
                  }
              }
          };  
        
        final SimpleComposite<Document> composite = new DefaultSimpleComposite<Document>(finder);
        
        return new ProxyLookup(Lookups.fixed(composite), super.createLookup());
      }
  }
